/** Source code example for "A Practical Introduction to Data
    Structures and Algorithm Analysis, 3rd Edition (Java)" 
    by Clifford A. Shaffer
    Copyright 2008-2011 by Clifford A. Shaffer
*/

/** Dictionary implemented by unsorted array-based list. */
class UALdictionary<K, E> implements Dictionary<K, E> {

  private static final int defaultSize = 10; // Default size
  private AList<KVpair<K,E>> list;  // To store dictionary

  // Constructors
  UALdictionary() { this(defaultSize); }
  UALdictionary(int sz)
    { list = new AList<KVpair<K, E>>(sz); }

  public void clear() { list.clear(); }     // Reinitialize

  /** Insert an element: append to list */
  public void insert(K k, E e) {
    KVpair<K,E> temp = new KVpair<K,E>(k, e);
    list.append(temp);
  }

  /** Use sequential search to find the element to remove */
  public E remove(K k) {
    E temp = find(k);
    if (temp != null) list.remove();
    return temp;
  }

  /** Remove the last element */
  public E removeAny() {
    if (size() != 0) {
      list.moveToEnd();
      list.prev();
      KVpair<K,E> e = list.remove();
      return e.value();
    }
    else return null;
  }

  // Find "k" using sequential search
  public E find(K k) {
    for(list.moveToStart(); list.currPos() < list.length();
        list.next()) {
      KVpair<K,E> temp = list.getValue();
      if (k == temp.key())
        return temp.value();
    }
    return null; // "k" does not appear in dictionary
  }
  public int size() // Return list size
    { return list.length(); }
}
